;;; Guile-Git --- GNU Guile bindings of libgit2
;;; Copyright © 2016 Amirouche Boubekki <amirouche@hypermove.net>
;;; Copyright © 2017 Erik Edrosa <erik.edrosa@gmail.com>
;;; Copyright © 2017 Ludovic Courtès <ludo@gnu.org>
;;;
;;; This file is part of Guile-Git.
;;;
;;; Guile-Git is free software; you can redistribute it and/or modify it
;;; under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or
;;; (at your option) any later version.
;;;
;;; Guile-Git is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with Guile-Git.  If not, see <http://www.gnu.org/licenses/>.

(define-module (tests reference)
  #:use-module (srfi srfi-64))

(use-modules (tests helpers))
(use-modules (git))


(test-begin "reference")

(libgit2-init!)

(with-repository "simple" directory

  (test-equal "reference-name"
    "refs/heads/master"
    (let* ((repository (repository-open directory)))
      (reference-name (repository-head repository))))

  (test-equal "reference-target"
    "3f848a1a52416ac99a5c5bf2e6bd55eb7b99d55b"
    (let* ((repository (repository-open directory)))
      (oid->string (reference-target (repository-head repository)))))

  (test-equal "reference-name->oid"
    "3f848a1a52416ac99a5c5bf2e6bd55eb7b99d55b"
    (let* ((repository (repository-open directory)))
      (oid->string (reference-name->oid repository "refs/heads/master"))))

  (test-equal "reference-shorthand"
    "master"
    (let* ((repository (repository-open directory)))
      (reference-shorthand (repository-head repository))))

  (test-equal "reference-lookup by name"
    '("refs/heads/master"
      "3f848a1a52416ac99a5c5bf2e6bd55eb7b99d55b"
      "master")
    (let* ((repository (repository-open directory))
           (reference (reference-lookup repository "refs/heads/master")))
      (list (reference-name reference)
            (oid->string (reference-target reference))
            (reference-shorthand reference))))

  (test-equal "reference-lookup non-existing"
    (list GIT_ENOTFOUND
          GITERR_REFERENCE)
    (let* ((repository (repository-open directory))
           (non-existing-ref "refs/non-existing"))
      (catch 'git-error
        (lambda ()
          (reference-lookup repository non-existing-ref))
        (lambda (key err)
          (list (git-error-code err)
                (git-error-class err))))))

  (test-equal "reference-create"
    (list GIT_ENOTFOUND
          GITERR_REFERENCE
          "3f848a1a52416ac99a5c5bf2e6bd55eb7b99d55b")
    (let* ((repository (repository-open directory))
           (new-ref-name "refs/pulls/1")
           (id (string->oid "3f848a1a52416ac99a5c5bf2e6bd55eb7b99d55b"))
           (error (catch 'git-error
                    (lambda ()
                      (reference-lookup repository new-ref-name))
                    (lambda (key err)
                      (list (git-error-code err)
                            (git-error-class err))))))
      (reference-create repository new-ref-name id)
      (append error
              (list
               (oid->string
                (reference-target
                 (reference-lookup repository new-ref-name)))))))

  (test-equal "reference-symbolic-create"
    (list GIT_ENOTFOUND
          GITERR_REFERENCE
          "refs/pulls/2"
          "3f848a1a52416ac99a5c5bf2e6bd55eb7b99d55b")
    (let* ((repository (repository-open directory))
           (new-ref-name "refs/pulls/2")
           (target "refs/heads/master")
           (error (catch 'git-error
                    (lambda ()
                      (reference-lookup repository new-ref-name))
                    (lambda (key err)
                      (list (git-error-code err)
                            (git-error-class err))))))
      (reference-symbolic-create repository new-ref-name target)
      (append error
              (list
               (reference-name (reference-lookup repository new-ref-name))
               (oid->string (reference-name->oid repository new-ref-name)))))))

(libgit2-shutdown!)

(test-end)
