/* Guile-Git --- GNU Guile bindings of libgit2
   Copyright © 2016 Amirouche Boubekki <amirouche@hypermove.net>
   Copyright © 2016, 2017 Erik Edrosa <erik.edrosa@gmail.com>

   This file is part of Guile-Git.

   Guile-Git is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   Guile-Git is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Guile-Git.  If not, see <http://www.gnu.org/licenses/>.  */

body {
    background: url('bg.png');
    font-family: mono;
    color: #FEFEFE;
    font-size: 12px;
    line-height: 1.6em;
    padding: 15px;
}

h1, h2, h3, h4, h5, h6 {
    font-family: mono;
    text-shadow: 2px 2px 0px black;
    margin: 0px;
    padding: 0px;
    line-height: 1em;
}

p {
    margin: 0px;
    padding: 0px;
}

a:link,
a:visited,
a:active {
    color: white;
    text-decoration: none;
}

body > div {
    background: hsla(0, 0%, 0%, 0.3);
    background: linear-gradient(0deg, hsla(0, 0%, 0%, 0.1) 5px, hsla(0, 0%, 100%, 0.1));    
    border-bottom: 1px solid hsla(0, 0%, 0%, 1);
    border-radius: 3px;
    border-right: 1px solid hsla(0, 0%, 0%, 1);    
    box-shadow: -1px -1px 0px hsla(0, 0%, 100%, 0.3), 1px 1px 2px black;
    display: block;
    margin: auto;
    margin-bottom: 15px;    
    max-width: 500px;
    padding: 15px;
    text-align: justify;
}

#container{
    padding: 0px;
    background: none;
}

#container .repositories a {
    display: block;
    padding: 30px;
    border-bottom: 1px solid hsla(0, 0%, 0%, 0.7);
    background: linear-gradient(0deg, hsla(0, 0%, 0%, 0.1) 5px, hsla(0, 0%, 100%, 0.1));    
}

#container .repositories a:last-child {
    border-bottom: none;
}

#container .repositories a p {
    margin-top: 10px;
}

.branches {
    padding: 15px;
    background: linear-gradient(0deg, hsla(0, 0%, 0%, 0.1) 5px, hsla(0, 0%, 100%, 0.1));
}

.branches > ul {
    list-style: outside none none;
    padding-left: 10px;
}
